
/******************************************************************************
* MODULE     : content.cpp
* DESCRIPTION: Wrapper for symbolic type without header dependencies
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "base.hpp"
#include "symbolic.hpp"
#include "rational.hpp"
namespace mml {

/******************************************************************************
* Internal representation
******************************************************************************/

class content_rep: public rep_struct {
public:
  vector<symbolic> v;
  inline content_rep (): v () {}
  inline content_rep (const symbolic& s): v (vec (s)) {}
  friend class content;
};

symbolic
as_symbolic (const content& c) {
  assert (dim (c->v) == 1);
  return c->v[0];
}

/******************************************************************************
* Constructors, destructors and accessors
******************************************************************************/

INDIRECT_STATIC_IMPL (content, content_rep)
content::content (): rep (new content_rep ()) {}
content::content (int i): rep (new content_rep (symbolic (i))) {}
content::content (char* s): rep (new content_rep (symbolic (s))) {}
content::content (const string& s): rep (new content_rep (symbolic (s))) {}
content::content (const integer& i): rep (new content_rep (symbolic (i))) {}
content::content (const rational& r): rep (new content_rep (symbolic (r))) {}
content::content (const symbolic& s): rep (new content_rep (s)) {}

/******************************************************************************
* Operations
******************************************************************************/

void lisp_print (ostream& out, const content& c) {
  lisp_print (out, as_symbolic (c)); }
ostream& operator << (ostream& out, const content& c) {
  return out << as_symbolic (c); }

content& operator << (content& c1, const content& c2) {
  assert (c1->ref_count == 1);
  c1->v << c2->v;
  return c1; }
content apply (const content& c1) {
  return symbolic_compound (c1->v); }
content apply (const content& c1, const content& c2) {
  return symbolic_compound (append (c1->v, c2->v)); }
content apply (const content& c1, const content& c2, const content& c3) {
  return symbolic_compound (append (c1->v, append (c2->v, c3->v))); }

content tuple (const content& c) {
  return apply (SYMB_TUPLE, c); }
content vtuple (const content& c) {
  return apply (SYMB_VTUPLE, c); }
content sqtuple (const content& c) {
  return apply (SYMB_SQTUPLE, c); }

content operator - (const content& c1) {
  return -as_symbolic (c1); }
content operator + (const content& c1, const content& c2) {
  return as_symbolic (c1) + as_symbolic (c2); }
content operator - (const content& c1, const content& c2) {
  return as_symbolic (c1) - as_symbolic (c2); }
content operator * (const content& c1, const content& c2) {
  return as_symbolic (c1) * as_symbolic (c2); }
content operator / (const content& c1, const content& c2) {
  return as_symbolic (c1) / as_symbolic (c2); }
content pow (const content& c1, const content& c2) {
  return pow (as_symbolic (c1), as_symbolic (c2)); }

} // namespace mml
