
/******************************************************************************
* MODULE     : symbol.hpp
* DESCRIPTION: A symbol points to another object but is stored only once
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_SYMBOL_HPP
#define __MML_SYMBOL_HPP
#include "table.hpp"
namespace mml {
#define TMPL_DEF template<typename C>
#define TMPL template<typename C>
#define Symbol symbol<C>
#define Symbol_rep symbol_rep<C>
TMPL class symbol_rep;
TMPL class symbol;

/******************************************************************************
* Symbol class and its representation class
******************************************************************************/

TMPL_DEF
class symbol_rep REP_STRUCT {
public: // FIXME: should become private
  static table<Symbol_rep*,C> symbol_entries;
private:
  const C ref;
public:
  Symbol_rep (const C& ref2): ref (ref2) { symbol_entries (ref)= this; }
  ~Symbol_rep () { symbol_entries->reset (ref); }
  friend class Symbol;
};

TMPL_DEF
class symbol {
INDIRECT_PROTO_1 (symbol, symbol_rep, C)
public:
  symbol (const C& ref) {
    rep= Symbol_rep::symbol_entries[ref];
    if (rep == (Symbol_rep*) NULL) rep= new Symbol_rep (ref);
    else rep->ref_count++;
    // cout << "rep  = " << rep << "\n";
    // cout << "count= " << rep->ref_count << "\n";
  }
  inline const C& operator * () const { return rep->ref; };
  inline friend nat hash (const Symbol& s) { return (nat) s.rep; }
  bool operator == (const Symbol& s) { return rep == s.rep; }
  bool operator != (const Symbol& s) { return rep != s.rep; }
};
INDIRECT_IMPL_1 (symbol, symbol_rep, typename C, C)

/******************************************************************************
* Printing symbols
******************************************************************************/

TMPL table<Symbol_rep*,C> Symbol_rep::symbol_entries ((Symbol_rep*) NULL);
TMPL content as_content (const Symbol& s) {
  return as_content ("symbol", *s); }
PRINT_SUGAR(TMPL,Symbol)

TMPL void
show_symbol_entries () {
  cout << "symbols\t= " << Symbol_rep::symbol_entries << "\n";
}

#undef TMPL_DEF
#undef TMPL
#undef Symbol
#undef Symbol_rep
} // namespace mml
#endif // __MML_SYMBOL_HPP
