
/******************************************************************************
* MODULE     : polynomial.hpp
* DESCRIPTION: Generic univariate polynomials
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_POLYNOMIAL_HPP
#define __MML_POLYNOMIAL_HPP
#include "string.hpp"
#include "iterator.hpp"
namespace mml {
#define TMPL_DEF template<typename C, typename V=dense<naive> >
#define TMPL template<typename C, typename V>
#define Polynomial polynomial<C,V>
TMPL class polynomial;

/******************************************************************************
* Abstract specification of dense polynomials
******************************************************************************/

TMPL_DEF class polynomial_rep REP_STRUCT {};

TMPL_DEF
class polynomial {
INDIRECT_PROTO_2 (polynomial, polynomial_rep, C, V)
public:
  polynomial ();
  polynomial (int x);
  polynomial (const C& x, nat n=0);
  polynomial (const iterator<C>& it);
  C operator[] (nat i);
};
INDIRECT_IMPL_2 (polynomial, polynomial_rep, typename C, C, typename V, V)

TMPL int deg (const Polynomial& P);

/******************************************************************************
* Polynomial iteration
******************************************************************************/

TMPL_DEF
class polynomial_iterator_rep: public iterator_rep<C> {
  Polynomial P;  // the polynomial to traverse
  nat i;         // current position
protected:
  bool is_busy () { return ((int) i) <= deg (P); }
  void advance () { i++; }
  C current () { return P[i]; }
public:
  polynomial_iterator_rep (const Polynomial& P2): P(P2), i(0) {}
};

TMPL iterator<C>
iterate (const Polynomial& P) {
  return iterator<C> (new polynomial_iterator_rep<C,V> (P));
}

/******************************************************************************
* Generic output routines for dense polynomials
******************************************************************************/

TMPL string var (const Polynomial& P) {
  (void) P; return the_polynomial_var; }

TMPL content
as_content (const Polynomial& P) {
  int i, len= deg(P);
  content s= 0;
  for (i=len; i>=0; i--)
    s= s + as_content (P[i]) * pow (as_content (var (P)), content (i));
  return s;
}

PRINT_SUGAR(TMPL,Polynomial)

#undef TMPL_DEF
#undef TMPL
#undef Polynomial
} // namespace mml
#endif // __MML_POLYNOMIAL_HPP
