
/******************************************************************************
* MODULE     : symbolic_aux.hpp
* DESCRIPTION: Auxiliary code for storing eacg symbolic expression only once
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __SYMBOLIC_AUX_HPP
#define __SYMBOLIC_AUX_HPP
#include "symbolic.hpp"
#include "table.hpp"
namespace mml {

typedef symbolic_rep* symbolic_ptr;

class symbolic_aux {
  symbolic_ptr rep;
public:
  inline symbolic_aux (symbolic_ptr rep2): rep (rep2) {}
  inline symbolic_aux (const symbolic_aux& s): rep (s.rep) {}
  inline friend bool
  operator == (const symbolic_aux& s1, const symbolic_aux& s2) {
    return s1.rep->identical (s2.rep); }
  inline friend bool
  operator != (const symbolic_aux& s1, const symbolic_aux& s2) {
    return !s1.rep->identical (s2.rep); }
  inline friend nat hash (const symbolic_aux& s) {
    return s.rep->hash (); }
  inline friend symbolic_ptr memorize (symbolic_ptr rep);
  friend class symbolic;
};

extern table<symbolic_ptr,symbolic_aux> symbolic_entries;

inline symbolic_ptr
memorize (symbolic_ptr rep) {
  // assert (rep->ref_count == 1);
  symbolic_ptr& rep2= symbolic_entries (symbolic_aux (rep));
  // assert (rep->ref_count == 1);
  // cout << "  Memorize " << ", " << N(symbolic_entries) << rep << " -> ";
  // if (rep2 == NULL) cout << "Interned\n"; else cout << "Duplicate\n";
  if (rep2 == NULL) rep2= rep;
  else { delete rep; INC_COUNT (rep2); }
  return rep2;
}

inline void
forget (symbolic_ptr rep) {
  if (rep != NULL && rep->ref_count == 1) {
    symbolic_entries->reset (symbolic_aux (rep));
    // cout << "  Destroy " << (void*) rep
    // << ", " << N(symbolic_entries) << "\n";
  }
}

} // namespace mml
#endif // __SYMBOLIC_AUX_HPP
